@@ -1,10 +1,10 @@
-use ruff_diagnostics::{AlwaysFixableViolation, Diagnostic, Edit, Fix};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::{self as ast, CmpOp, Expr};
 use ruff_python_semantic::analyze::typing;
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
+use crate::{AlwaysFixableViolation, Edit, Fix};
 
 /// ## What it does
 /// Checks for membership tests on `list` and `tuple` literals.
@@ -25,28 +25,30 @@ use crate::checkers::ast::Checker;
 ///
 /// ## Fix safety
 /// This rule's fix is marked as unsafe, as the use of a `set` literal will
-/// error at runtime if the sequence contains unhashable elements (like lists
-/// or dictionaries). While Ruff will attempt to infer the hashability of the
-/// elements, it may not always be able to do so.
+/// error at runtime if either the element being tested for membership (the
+/// left-hand side) or any element of the sequence (the right-hand side)
+/// is unhashable (like lists or dictionaries). While Ruff will attempt to
+/// infer the hashability of both sides and skip the fix when it can determine
+/// that either side is unhashable, it may not always be able to do so.
 ///
 /// ## References
 /// - [What’s New In Python 3.2](https://docs.python.org/3/whatsnew/3.2.html#optimizations)
-#[violation]
-pub struct LiteralMembership;
+#[derive(ViolationMetadata)]
+pub(crate) struct LiteralMembership;
 
 impl AlwaysFixableViolation for LiteralMembership {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Use a `set` literal when testing for membership")
+        "Use a set literal when testing for membership".to_string()
     }
 
     fn fix_title(&self) -> String {
-        format!("Convert to `set`")
+        "Convert to `set`".to_string()
     }
 }
 
 /// PLR6201
-pub(crate) fn literal_membership(checker: &mut Checker, compare: &ast::ExprCompare) {
+pub(crate) fn literal_membership(checker: &Checker, compare: &ast::ExprCompare) {
     let [op] = &*compare.ops else {
         return;
     };
@@ -65,6 +67,11 @@ pub(crate) fn literal_membership(checker: &mut Checker, compare: &ast::ExprCompa
         _ => return,
     };
 
+    // Skip empty collections (#15729).
+    if elts.is_empty() {
+        return;
+    }
+
     // If `left`, or any of the elements in `right`, are known to _not_ be hashable, return.
     if std::iter::once(compare.left.as_ref())
         .chain(elts)
@@ -96,7 +103,7 @@ pub(crate) fn literal_membership(checker: &mut Checker, compare: &ast::ExprCompa
         return;
     }
 
-    let mut diagnostic = Diagnostic::new(LiteralMembership, right.range());
+    let mut diagnostic = checker.report_diagnostic(LiteralMembership, right.range());
 
     let literal = checker.locator().slice(right);
     let set = format!("{{{}}}", &literal[1..literal.len() - 1]);
@@ -104,6 +111,4 @@ pub(crate) fn literal_membership(checker: &mut Checker, compare: &ast::ExprCompa
         set,
         right.range(),
     )));
-
-    checker.diagnostics.push(diagnostic);
 }
