@@ -1,13 +1,14 @@
-use anyhow::{bail, Result};
-use ruff_diagnostics::{Diagnostic, Edit, Fix, FixAvailability, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use anyhow::{Result, bail};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::comparable::ComparableExpr;
 use ruff_python_ast::helpers::any_over_expr;
+use ruff_python_ast::name::Name;
 use ruff_python_ast::{self as ast, Expr};
 use ruff_text_size::{Ranged, TextRange};
 
 use crate::checkers::ast::Checker;
 use crate::importer::ImportRequest;
+use crate::{Edit, Fix, FixAvailability, Violation};
 
 /// ## What it does
 /// Checks for generator expressions, list and set comprehensions that can
@@ -28,18 +29,7 @@ use crate::importer::ImportRequest;
 ///
 /// ## Example
 /// ```python
-/// scores = [85, 100, 60]
-/// passing_scores = [60, 80, 70]
-///
-///
-/// def passed_test(score: int, passing_score: int) -> bool:
-///     return score >= passing_score
-///
-///
-/// passed_all_tests = all(
-///     passed_test(score, passing_score)
-///     for score, passing_score in zip(scores, passing_scores)
-/// )
+/// all(predicate(a, b) for a, b in some_iterable)
 /// ```
 ///
 /// Use instead:
@@ -47,15 +37,7 @@ use crate::importer::ImportRequest;
 /// from itertools import starmap
 ///
 ///
-/// scores = [85, 100, 60]
-/// passing_scores = [60, 80, 70]
-///
-///
-/// def passed_test(score: int, passing_score: int) -> bool:
-///     return score >= passing_score
-///
-///
-/// passed_all_tests = all(starmap(passed_test, zip(scores, passing_scores)))
+/// all(starmap(predicate, some_iterable))
 /// ```
 ///
 /// ## References
@@ -63,26 +45,26 @@ use crate::importer::ImportRequest;
 ///
 /// [PEP 709]: https://peps.python.org/pep-0709/
 /// [#7771]: https://github.com/astral-sh/ruff/issues/7771
-#[violation]
-pub struct ReimplementedStarmap;
+#[derive(ViolationMetadata)]
+pub(crate) struct ReimplementedStarmap;
 
 impl Violation for ReimplementedStarmap {
     const FIX_AVAILABILITY: FixAvailability = FixAvailability::Sometimes;
 
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Use `itertools.starmap` instead of the generator")
+        "Use `itertools.starmap` instead of the generator".to_string()
     }
 
     fn fix_title(&self) -> Option<String> {
-        Some(format!("Replace with `itertools.starmap`"))
+        Some("Replace with `itertools.starmap`".to_string())
     }
 }
 
 /// FURB140
-pub(crate) fn reimplemented_starmap(checker: &mut Checker, target: &StarmapCandidate) {
+pub(crate) fn reimplemented_starmap(checker: &Checker, target: &StarmapCandidate) {
     // Generator should have exactly one comprehension.
-    let [comprehension @ ast::Comprehension { .. }] = target.generators() else {
+    let [comprehension] = target.generators() else {
         return;
     };
 
@@ -133,9 +115,9 @@ pub(crate) fn reimplemented_starmap(checker: &mut Checker, target: &StarmapCandi
         }
         // Ex) `f(x, y, z) for x, y, z in iter`
         ComprehensionTarget::Tuple(tuple) => {
-            if tuple.elts.len() != args.len()
-                || !std::iter::zip(&tuple.elts, args)
-                    .all(|(x, y)| ComparableExpr::from(x) == ComparableExpr::from(y))
+            if tuple.len() != args.len()
+                || std::iter::zip(tuple, args)
+                    .any(|(x, y)| ComparableExpr::from(x) != ComparableExpr::from(y))
             {
                 return;
             }
@@ -143,16 +125,15 @@ pub(crate) fn reimplemented_starmap(checker: &mut Checker, target: &StarmapCandi
             // If any of the members are used outside the function call, we can't replace it.
             if any_over_expr(func, &|expr| {
                 tuple
-                    .elts
                     .iter()
-                    .any(|elt| ComparableExpr::from(expr) == ComparableExpr::from(elt))
+                    .any(|elem| ComparableExpr::from(expr) == ComparableExpr::from(elem))
             }) {
                 return;
             }
         }
     }
 
-    let mut diagnostic = Diagnostic::new(ReimplementedStarmap, target.range());
+    let mut diagnostic = checker.report_diagnostic(ReimplementedStarmap, target.range());
     diagnostic.try_set_fix(|| {
         // Import `starmap` from `itertools`.
         let (import_edit, starmap_name) = checker.importer().get_or_import_symbol(
@@ -165,12 +146,16 @@ pub(crate) fn reimplemented_starmap(checker: &mut Checker, target: &StarmapCandi
         // - For list and set comprehensions, we'd want to wrap it with `list` and `set`
         //   correspondingly.
         let main_edit = Edit::range_replacement(
-            target.try_make_suggestion(starmap_name, &comprehension.iter, func, checker)?,
+            target.try_make_suggestion(
+                Name::from(starmap_name),
+                &comprehension.iter,
+                func,
+                checker,
+            )?,
             target.range(),
         );
         Ok(Fix::safe_edits(import_edit, [main_edit]))
     });
-    checker.diagnostics.push(diagnostic);
 }
 
 /// An enum for a node that can be considered a candidate for replacement with `starmap`.
@@ -231,7 +216,7 @@ impl StarmapCandidate<'_> {
     /// Try to produce a fix suggestion transforming this node into a call to `starmap`.
     pub(crate) fn try_make_suggestion(
         &self,
-        name: String,
+        name: Name,
         iter: &Expr,
         func: &Expr,
         checker: &Checker,
@@ -260,7 +245,7 @@ impl StarmapCandidate<'_> {
                 // ```python
                 // list(itertools.starmap(foo, iter))
                 // ```
-                try_construct_call(name, iter, func, "list", checker)
+                try_construct_call(name, iter, func, Name::new_static("list"), checker)
             }
             Self::SetComp(_) => {
                 // For set comprehensions, we replace:
@@ -272,7 +257,7 @@ impl StarmapCandidate<'_> {
                 // ```python
                 // set(itertools.starmap(foo, iter))
                 // ```
-                try_construct_call(name, iter, func, "set", checker)
+                try_construct_call(name, iter, func, Name::new_static("set"), checker)
             }
         }
     }
@@ -280,15 +265,15 @@ impl StarmapCandidate<'_> {
 
 /// Try constructing the call to `itertools.starmap` and wrapping it with the given builtin.
 fn try_construct_call(
-    name: String,
+    name: Name,
     iter: &Expr,
     func: &Expr,
-    builtin: &str,
+    builtin: Name,
     checker: &Checker,
 ) -> Result<String> {
     // We can only do our fix if `builtin` identifier is still bound to
     // the built-in type.
-    if !checker.semantic().has_builtin_binding(builtin) {
+    if !checker.semantic().has_builtin_binding(&builtin) {
         bail!("Can't use built-in `{builtin}` constructor")
     }
 
@@ -308,11 +293,12 @@ fn try_construct_call(
 }
 
 /// Construct the call to `itertools.starmap` for suggestion.
-fn construct_starmap_call(starmap_binding: String, iter: &Expr, func: &Expr) -> ast::ExprCall {
+fn construct_starmap_call(starmap_binding: Name, iter: &Expr, func: &Expr) -> ast::ExprCall {
     let starmap = ast::ExprName {
         id: starmap_binding,
         ctx: ast::ExprContext::Load,
         range: TextRange::default(),
+        node_index: ruff_python_ast::AtomicNodeIndex::NONE,
     };
     ast::ExprCall {
         func: Box::new(starmap.into()),
@@ -320,17 +306,20 @@ fn construct_starmap_call(starmap_binding: String, iter: &Expr, func: &Expr) ->
             args: Box::from([func.clone(), iter.clone()]),
             keywords: Box::from([]),
             range: TextRange::default(),
+            node_index: ruff_python_ast::AtomicNodeIndex::NONE,
         },
         range: TextRange::default(),
+        node_index: ruff_python_ast::AtomicNodeIndex::NONE,
     }
 }
 
 /// Wrap given function call with yet another call.
-fn wrap_with_call_to(call: ast::ExprCall, func_name: &str) -> ast::ExprCall {
+fn wrap_with_call_to(call: ast::ExprCall, func_name: Name) -> ast::ExprCall {
     let name = ast::ExprName {
-        id: func_name.to_string(),
+        id: func_name,
         ctx: ast::ExprContext::Load,
         range: TextRange::default(),
+        node_index: ruff_python_ast::AtomicNodeIndex::NONE,
     };
     ast::ExprCall {
         func: Box::new(name.into()),
@@ -338,8 +327,10 @@ fn wrap_with_call_to(call: ast::ExprCall, func_name: &str) -> ast::ExprCall {
             args: Box::from([call.into()]),
             keywords: Box::from([]),
             range: TextRange::default(),
+            node_index: ruff_python_ast::AtomicNodeIndex::NONE,
         },
         range: TextRange::default(),
+        node_index: ruff_python_ast::AtomicNodeIndex::NONE,
     }
 }
 
@@ -352,7 +343,9 @@ enum ComprehensionTarget<'a> {
 }
 
 /// Extract the target from the comprehension (e.g., `(x, y, z)` in `(x, y, z, ...) in iter`).
-fn match_comprehension_target(comprehension: &ast::Comprehension) -> Option<ComprehensionTarget> {
+fn match_comprehension_target(
+    comprehension: &ast::Comprehension,
+) -> Option<ComprehensionTarget<'_>> {
     if comprehension.is_async || !comprehension.ifs.is_empty() {
         return None;
     }
