@@ -1,5 +1,7 @@
 use std::any::Any;
+use std::fmt;
 use std::fmt::Display;
+use std::panic::RefUnwindSafe;
 use std::sync::Arc;
 
 use lsp_types::Url;
@@ -7,8 +9,8 @@ use ruff_db::file_revision::FileRevision;
 use ruff_db::files::{File, FilePath};
 use ruff_db::system::walk_directory::WalkDirectoryBuilder;
 use ruff_db::system::{
-    CaseSensitivity, DirectoryEntry, FileType, GlobError, Metadata, OsSystem, PatternError, Result,
-    System, SystemPath, SystemPathBuf, SystemVirtualPath, SystemVirtualPathBuf, WritableSystem,
+    CaseSensitivity, DirectoryEntry, FileType, GlobError, Metadata, PatternError, Result, System,
+    SystemPath, SystemPathBuf, SystemVirtualPath, SystemVirtualPathBuf, WritableSystem,
 };
 use ruff_notebook::{Notebook, NotebookError};
 use ty_python_semantic::Db;
@@ -97,7 +99,16 @@ impl AnySystemPath {
     }
 }
 
-#[derive(Debug)]
+impl fmt::Display for AnySystemPath {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            AnySystemPath::System(system_path) => write!(f, "{system_path}"),
+            AnySystemPath::SystemVirtual(virtual_path) => write!(f, "{virtual_path}"),
+        }
+    }
+}
+
+#[derive(Debug, Clone)]
 pub(crate) struct LSPSystem {
     /// A read-only copy of the index where the server stores all the open documents and settings.
     ///
@@ -108,18 +119,21 @@ pub(crate) struct LSPSystem {
     /// [`index_mut`]: crate::Session::index_mut
     index: Option<Arc<Index>>,
 
-    /// A system implementation that uses the local file system.
-    os_system: OsSystem,
+    /// A native system implementation.
+    ///
+    /// This is used to delegate method calls that are not handled by the LSP system. It is also
+    /// used as a fallback when the documents are not found in the LSP index.
+    native_system: Arc<dyn System + 'static + Send + Sync + RefUnwindSafe>,
 }
 
 impl LSPSystem {
-    pub(crate) fn new(index: Arc<Index>) -> Self {
-        let cwd = std::env::current_dir().unwrap();
-        let os_system = OsSystem::new(SystemPathBuf::from_path_buf(cwd).unwrap());
-
+    pub(crate) fn new(
+        index: Arc<Index>,
+        native_system: Arc<dyn System + 'static + Send + Sync + RefUnwindSafe>,
+    ) -> Self {
         Self {
             index: Some(index),
-            os_system,
+            native_system,
         }
     }
 
@@ -145,7 +159,7 @@ impl LSPSystem {
     fn make_document_ref(&self, path: AnySystemPath) -> Option<DocumentQuery> {
         let index = self.index();
         let key = DocumentKey::from_path(path);
-        index.make_document_ref(&key)
+        index.make_document_ref(key).ok()
     }
 
     fn system_path_to_document_ref(&self, path: &SystemPath) -> Option<DocumentQuery> {
@@ -173,16 +187,16 @@ impl System for LSPSystem {
                 FileType::File,
             ))
         } else {
-            self.os_system.path_metadata(path)
+            self.native_system.path_metadata(path)
         }
     }
 
     fn canonicalize_path(&self, path: &SystemPath) -> Result<SystemPathBuf> {
-        self.os_system.canonicalize_path(path)
+        self.native_system.canonicalize_path(path)
     }
 
     fn path_exists_case_sensitive(&self, path: &SystemPath, prefix: &SystemPath) -> bool {
-        self.os_system.path_exists_case_sensitive(path, prefix)
+        self.native_system.path_exists_case_sensitive(path, prefix)
     }
 
     fn read_to_string(&self, path: &SystemPath) -> Result<String> {
@@ -190,7 +204,7 @@ impl System for LSPSystem {
 
         match document {
             Some(DocumentQuery::Text { document, .. }) => Ok(document.contents().to_string()),
-            _ => self.os_system.read_to_string(path),
+            _ => self.native_system.read_to_string(path),
         }
     }
 
@@ -202,7 +216,7 @@ impl System for LSPSystem {
                 Notebook::from_source_code(document.contents())
             }
             Some(DocumentQuery::Notebook { notebook, .. }) => Ok(notebook.make_ruff_notebook()),
-            None => self.os_system.read_to_notebook(path),
+            None => self.native_system.read_to_notebook(path),
         }
     }
 
@@ -233,26 +247,26 @@ impl System for LSPSystem {
     }
 
     fn current_directory(&self) -> &SystemPath {
-        self.os_system.current_directory()
+        self.native_system.current_directory()
     }
 
     fn user_config_directory(&self) -> Option<SystemPathBuf> {
-        self.os_system.user_config_directory()
+        self.native_system.user_config_directory()
     }
 
     fn cache_dir(&self) -> Option<SystemPathBuf> {
-        self.os_system.cache_dir()
+        self.native_system.cache_dir()
     }
 
     fn read_directory<'a>(
         &'a self,
         path: &SystemPath,
     ) -> Result<Box<dyn Iterator<Item = Result<DirectoryEntry>> + 'a>> {
-        self.os_system.read_directory(path)
+        self.native_system.read_directory(path)
     }
 
     fn walk_directory(&self, path: &SystemPath) -> WalkDirectoryBuilder {
-        self.os_system.walk_directory(path)
+        self.native_system.walk_directory(path)
     }
 
     fn glob(
@@ -262,11 +276,11 @@ impl System for LSPSystem {
         Box<dyn Iterator<Item = std::result::Result<SystemPathBuf, GlobError>> + '_>,
         PatternError,
     > {
-        self.os_system.glob(pattern)
+        self.native_system.glob(pattern)
     }
 
     fn as_writable(&self) -> Option<&dyn WritableSystem> {
-        self.os_system.as_writable()
+        self.native_system.as_writable()
     }
 
     fn as_any(&self) -> &dyn Any {
@@ -278,11 +292,15 @@ impl System for LSPSystem {
     }
 
     fn case_sensitivity(&self) -> CaseSensitivity {
-        self.os_system.case_sensitivity()
+        self.native_system.case_sensitivity()
     }
 
     fn env_var(&self, name: &str) -> std::result::Result<String, std::env::VarError> {
-        self.os_system.env_var(name)
+        self.native_system.env_var(name)
+    }
+
+    fn dyn_clone(&self) -> Box<dyn System> {
+        Box::new(self.clone())
     }
 }
 
